#ifndef AWALI_ALGOS_COMPLEMENT_HH
# define AWALI_ALGOS_COMPLEMENT_HH

# include <set>

#include <awali/sttc/algos/copy.hh>
#include <awali/sttc/algos/is-complete.hh>
#include <awali/sttc/algos/determinize.hh>
#include <awali/sttc/misc/raise.hh>
#include <awali/sttc/weightset/fwd.hh> // b

namespace awali { namespace sttc {


  /*------------------------.
  | complement(automaton).  |
  `------------------------*/

  template <typename Aut>
  void
  complement_here(Aut& aut)
  {
    static_assert(labelset_t_of<Aut>::is_free(),
                  "requires free labelset");
    static_assert(std::is_same<weightset_t_of<Aut>, b>::value,
                  "requires Boolean weights");

    // using automaton_t = Aut;

    require(is_deterministic(aut),
            "complement: requires a deterministic automaton");
    require(is_complete(aut),
            "complement: requires a complete automaton");

    // The final states of aut.
    std::set<state_t> finals;
    for (auto t: aut->final_transitions())
      finals.insert(aut->src_of(t));

    // Complement.
    for (auto s: aut->states())
      if (!aut->is_final(s))
        aut->set_final(s);
      else
        aut->unset_final(s);
  }

  template <typename Aut>
  auto
  complement(const Aut& aut, bool history=true)
    -> decltype(copy(aut))
  {
    auto res = copy(aut, history);
    complement_here(res);
    return res;
  }

}}//end of ns awali::stc

#endif // !AWALI_ALGOS_COMPLEMENT_HH
