#ifndef AWALI_ALGOS_IS_COMPLETE_HH
# define AWALI_ALGOS_IS_COMPLETE_HH

# include <queue>
# include <set>

namespace awali { namespace sttc {

  /// Whether \a aut is complete.
  /// \pre \a aut is LTL
  template <typename Aut>
  bool is_complete(const Aut& aut)
  {
    static_assert(labelset_t_of<Aut>::is_free(),
                  "requires free labelset");

    if (aut->num_initials() == 0)
      return false;

    using label_set_t = std::set<typename labelset_t_of<Aut>::letter_t>;

    const auto& letters = aut->labelset()->genset();
    for (auto state : aut->states())
    {
      label_set_t missing_letters = {std::begin(letters), std::end(letters)};

      for (auto tr : aut->all_out(state))
        missing_letters.erase(aut->label_of(tr));

      if (!missing_letters.empty())
        return false;
    }

    return true;
  }

}}//end of ns awali::stc

#endif // !AWALI_ALGOS_IS_COMPLETE_HH
