#ifndef AWALI_ALGOS_MIN_QUOTIENT_HH
#define AWALI_ALGOS_MIN_QUOTIENT_HH

#include <awali/common/param.hh>
#include <awali/sttc/algos/quotient/moore_quotient.hh>
#include <awali/sttc/algos/quotient/hopcroft_quotient.hh>
#include <awali/sttc/algos/quotient/congruence_det.hh>
#include <awali/sttc/algos/merge.hh>

namespace awali {
  namespace sttc {
    template <typename Aut>
    Aut min_quotient(const Aut& aut, param_t algo=MOORE) {
      std::vector<std::vector<state_t> > equiv;
      std::vector<std::list<state_t> > equil;
      switch(algo) {
      case MOORE :
	moore_quotient(aut, equiv);
	return merge(aut, equiv);
      case HOPCROFT :
	hopcroft_quotient(aut, equil);
	return merge(aut, equil);
      default:
	raise("Quotient algo is either MOORE or HOPCROFT");
      }
    }

    //For deterministic automata only
    template <typename Aut>
    Aut minimize(const Aut& aut, param_t algo=MOORE) {
      std::vector<std::vector<state_t> > equiv;
      std::vector<std::list<state_t> > equil;
      switch(algo) {
      case MOORE :
	moore_det(aut, equiv);
	return merge(aut, equiv);
      case HOPCROFT :
	hopcroft_quotient(aut, equil, true);
	return merge(aut, equil);
      default:
	raise("Quotient algo is either MOORE or HOPCROFT");
      }
    }

    template <typename Aut>
    Aut min_quotient_det(const Aut& aut, param_t algo=MOORE) {
      return minimize(aut, algo);
    }

  }
}//end of ns awali::stc

#endif // !AWALI_ALGOS_MIN_QUOTIENT_HH
