#ifndef AWALI_STRING_HISTORY_HH
#define AWALI_STRING_HISTORY_HH

# include <string>
# include <map>
#include <awali/sttc/history/history.hh>

namespace awali {
  namespace sttc {

    //Make every state in relation with a string
    class string_history  : public history_base
    {
    private:
      using origins_t = std::map<state_t, std::string>;
      origins_t origins_;
    public:
      string_history()
      {}

      history_kind_t get_nature() const
      {
	return history_kind_t::STRING;
      }

      std::ostream&
      print_state_name(state_t s, std::ostream& o,
		       const std::string& fmt) const
      {
	auto i = origins().find(s);
	o << i->second;
	return o;
      }

      const std::string& get_state_name(state_t s) const
      {
	auto i = origins().find(s);
	return i->second;
      }

      const origins_t& origins() const
      {
	return origins_;
      }

      bool remove_history(state_t s) {
	return origins_.erase(s);
      };

      bool has_history(state_t s) const {
	return (origins_.find(s)!=origins_.end());
      }

      void
      add_state(state_t s,const std::string& str)
      {
	origins_[s] = str;
      }

      state_t get_state(state_t s) {
	throw std::runtime_error("Origin state not available");
      }

      std::vector<state_t> get_state_set(state_t s) {
	throw std::runtime_error("Origin state set not available");
      }
    };

  }
}//end of ns awali::stc

#endif // !AWALI_STRING_HISTORY_HH
